// TWERP - EZTWAIN.DLL demo application
// Copyright (C) 1998 by Dosadi.  All Rights Reserved.
//
// Created  05/11/94    spike
// Revised  06/23/94    spike - rev 1.0  alpha 1
// Revised  07/17/94    spike - rev 1.0  alpha 2 - removed use of Bitmap
// Revised  02/06/95    spike - rev 1.1  added Save As command
// Revised  05/03/95    spike - rev 1.2  added Options menu with PixelTypes
// Revised  04/02/97    spike - rev 1.4  single source for Win16 and Win32.
// Revised  08/21/97    spike - rev 1.5  eztwain 1.06, no real changes
// Revised  06/14/98    spike - rev 1.6  my own 'Dosadi' version
// Revised  12/11/98    spike - rev 1.7  for EZTWAIN 1.5
// Revised  08/30/00    spike - rev 2.6  call RegisterApp
// Revised  2001-03-05  spike - rev 2.8  updated to VC6.0

//---------- Includes

#include "eztwain.h"
#include "twain.h"
#include "resource.h"

//---------- Global variables

static HANDLE   	hInst;              // current instance
static HWND			hwndMain;           // main window handle

static HANDLE		g_hdib;             // current DIB, if any
static HPALETTE 	hpal;               // logical palette for same, if any

static unsigned     wPixTypes;          // enabled pixel types
static double       dResolution = -1.0; // requested resolution if any
static int			fHideUI;            // hide source U/I
static int          fFeeder;            // document feeder (-1, 0, 1)
static BOOL         bAutoFeed;          // use automatic feeding
static BOOL         bDuplex;
static BOOL         bIndicators = TRUE; // enable CAP_INDICATORS
static BOOL         bNoMessages;        // suppress error messages
static int          nXferMech = XFERMECH_NATIVE;    // transfer mode
static int          nFileFormat = TWFF_BMP;
static int          insideAcquire;      // doing an Acquire!
static HCURSOR		hcurWait;           // wait cursor (hourglass)

static char			szAppName[32];
static char			szMsg[256];


//---------- Forward function declarations

BOOL InitInstance(int CmdShow);
long FAR PASCAL MainWndProc(HWND, WORD, WPARAM, LONG);
BOOL FAR PASCAL AboutDlgProc(HWND, UINT, WPARAM, LPARAM);
void ResizeWindow(HWND hwnd, HANDLE hdib);
VOID DiscardImage(VOID);                       
void SetNewImage(HWND hwnd, HANDLE hdib);

//---------- Public functions

int PASCAL WinMain (HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR  lpszCmdLine,
                    int CmdShow)
{
    WNDCLASS	wc;
    MSG			msg;

    lpszCmdLine=lpszCmdLine;			// suppress no-use warning

    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

	LoadString(hInst, IDS_APPNAME, szAppName, sizeof szAppName);

	if (hPrevInstance) {
        // Don't allow multiple instances of this program.
        return FALSE;
    }

    // Define & register main window class
    wc.style         = 0;								// this class has no style!
    wc.lpfnWndProc   = (WNDPROC)MainWndProc;			// name of window proc
    wc.cbClsExtra    = 0;								// no extra bits
    wc.cbWndExtra    = 0;
    wc.hInstance     = hInst;							// handle to cur instance
    wc.hIcon         = LoadIcon( hInst, "TW_APP_ICO");
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = CreateSolidBrush(RGB(0, 0, 128));	// use a twerpy background
    wc.lpszMenuName  = "TW_App_Menu";					// class default menu
    wc.lpszClassName = "TW_App_MainWnd";				// class named

    if (!RegisterClass(&wc)) {
        return FALSE;
    }

    if (!InitInstance(CmdShow)) {
            return (FALSE);
    }

    TWAIN_RegisterApp(
	2, 6,               // version
	TWLG_USA,           // (human) language (use TWLG_xxx from TWAIN.H)
	TWCY_USA,           // country (use TWCY_xxx from TWAIN.H)
	"2.60",             // version info string e.g. "1.0b3 Beta release"
	"Dosadi",           // name of mfg/developer e.g. "Crazbat Software"
	"EZTWAIN",          // product family e.g. "BitStomper"
	"Twerp32");         // specific product e.g. "BitStomper Deluxe Pro"

    // The main loop of the program.  Get a Message; if it's not an
    // accelerator key, translate it and dispatch it to the application.
    while (GetMessage((LPMSG)&msg, NULL, 0, 0)) {
        if (TWAIN_MessageHook ((LPMSG)&msg)) {
            if (TWAIN_IsTransferReady()) {
                // Trigger an acquire
                PostMessage(hwndMain, WM_COMMAND, TW_APP_ACQUIRE, 0);
            }
        } else {
        	TranslateMessage ((LPMSG)&msg);
        	DispatchMessage ((LPMSG)&msg);
        }
    }
    return (msg.wParam);
} // WinMain




//---------------------------------------------------------------------------

BOOL InitInstance(int CmdShow)
{
    char		WindowTitle[64];

    hcurWait = LoadCursor(NULL, IDC_WAIT);

    LoadString(hInst, IDS_WINDOWTITLE, WindowTitle,  sizeof(WindowTitle));
    hwndMain = CreateWindow("TW_App_MainWnd", WindowTitle, WS_OVERLAPPEDWINDOW,
             0, 0, 500, 350, NULL, NULL, hInst, NULL);

    if (!hwndMain)
        return (FALSE);

	wPixTypes = TWAIN_ANYTYPE;

    ShowWindow (hwndMain, CmdShow);
    UpdateWindow (hwndMain);
    return (TRUE);
} // InitInstance


char* FFExtension(int nFF)
{
    switch (nFF) {
    case TWFF_TIFF:
        return ".tif";
    case TWFF_PICT:
        return ".pic";
    case TWFF_BMP:
        return ".bmp";
    case TWFF_XBM:
        return ".xbm";
    case TWFF_JFIF:
        return ".jpg";
    case TWFF_FPX:
        return ".fpx";
    case TWFF_TIFFMULTI:
        return ".mpt";
    case TWFF_PNG:
        return ".png";
    case TWFF_SPIFF:
        return ".spi";
    case TWFF_EXIF:
        return ".exi";
    default:
        return ".huh";
    } // switch
} // FFExtension


BOOL PromptForFilename(char* pszFile, int* pnFF)
// TRUE if OK, FALSE if cancelled.
{
    // prompt for filename
    OPENFILENAME ofn;
    int nExt;
    int nFF = *pnFF;
    
    memset(&ofn, 0, sizeof ofn);
    pszFile[0] = '\0';
    ofn.lStructSize = sizeof(OPENFILENAME);
    ofn.hwndOwner = NULL;
    ofn.lpstrFilter = "\
TIFF (*.tif)\0*.tif\0\
Macintosh Pict (*.pic)\0*.pic\0\
Windows Bitmap (*.bmp)\0*.bmp\0\
X-Windows Bitmap (*.xbm)\0*.xbm\0\
JPEG/JFIF (*.jpg)\0*.jpg\0\
Kodak FlashPix (*.fpx)\0*.fpx\0\
MultiPage TIFF (*.mpt)\0*.mpt\0\
PNG (*.png)\0*.png\0\
SPIFF (*.spi)\0*.spi\0\
EXIF (*.exi)\0*.exi\0\0";
    ofn.lpstrDefExt = FFExtension(nFF);
    ofn.nFilterIndex = nFF+1;
    ofn.lpstrFile= pszFile;
    ofn.nMaxFile = _MAX_PATH - 5;
    ofn.Flags = OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY |
        OFN_NOREADONLYRETURN;
    
    if (!GetSaveFileName(&ofn)) {
        return FALSE;					// user cancelled dialog
    }
    // Use file format chosen by user in dialog
    nFF = ofn.nFilterIndex-1;
    // supply default extension - GetSaveFileName doesn't seem to do it!
    nExt = ofn.nFileExtension;
    if (nExt==0 || !pszFile[nExt]) {
        // no extension
        lstrcat(pszFile, FFExtension(nFF));
    }
    /*
    wsprintf(szMsg, "format: %d\nfile:   %s", nFF, pszFile);
    MessageBox(NULL, szMsg, "Twerp", MB_ICONINFORMATION | MB_OK);
    */

    *pnFF = nFF;
    return TRUE;
} // PromptForFilename


static char szFilename[_MAX_PATH];
static int nFF = TWFF_BMP;

HANDLE AcquireInMode(HWND hwnd)
{
    HANDLE hdib = NULL;
    insideAcquire++;
    // Acquire the next image
    switch (nXferMech) {
    case XFERMECH_NATIVE:
        hdib = TWAIN_AcquireNative(hwnd, 0);
        break;
    case XFERMECH_MEMORY:
        hdib = TWAIN_AcquireMemory(hwnd);
        break;
    case XFERMECH_FILE:
        if (PromptForFilename(szFilename, &nFF)) {
            if (TWAIN_AcquireFile(hwnd, nFF, szFilename)) {
                hdib = DIB_LoadFromFilename(szFilename);
            }
        }
        break;
    } // switch
    insideAcquire--;
    return hdib;
}


BOOL CapabilityNegotiation(HWND hwnd)
{
    if (bDuplex && TWAIN_GetDuplexSupport() == 0) {
        MessageBox(hwnd,
            "Duplex: Device reports no duplex support.",
            szAppName, MB_ICONINFORMATION | MB_OK);
    }
    if (fFeeder) {
        // Feeder configured
        if (!TWAIN_HasFeeder()) {
            MessageBox(hwnd,
                "This source indicates it has no document feeder.",
                szAppName, MB_ICONINFORMATION | MB_OK);
            // Unconfigure the feeder
            fFeeder = 0;
        } else if (fFeeder < 0) {
            // Feeder disabled
            TWAIN_SelectFeeder(FALSE);
        } else {
            // Feeder enabled
            TWAIN_SelectFeeder(TRUE);
            // Turn autofeed on or off
            TWAIN_SetAutoFeed(bAutoFeed);
        }
    }
    TWAIN_NegotiatePixelTypes(wPixTypes);
    return TRUE;
} // CapabilityNegotiation


BOOL SynchronousAcquire(HWND hwnd)
{
    HANDLE hdib = NULL;

    // free up current image and palette if any
	DiscardImage();
    // clear the window during/before Acquire dialog
    InvalidateRect(hwnd, NULL, TRUE);
    TWAIN_SetHideUI(fHideUI);
    if (TWAIN_State() < TWAIN_SOURCE_OPEN) {
        if (!TWAIN_OpenDefaultSource()) {
            // unable to open source - probably barfed already.
            return FALSE;
        }
        // Now in State 4 (TWAIN_SOURCE_OPEN)
        CapabilityNegotiation(hwnd);
    }
    hdib = AcquireInMode(hwnd);
    if (hdib) {
        SetNewImage(hwnd, hdib);
    } else {
        if (TWAIN_UserClosedSource()) {
            TWAIN_ErrorBox("User closed DS");
        } else {
            TWAIN_ReportLastError(NULL);
        }
    }
    return (hdib != NULL);
} // SynchronousAcquire


void MultiPageAcquire(HWND hwnd)
{
    BOOL bMulti = TWAIN_GetMultiTransfer();
    TWAIN_SetMultiTransfer(1);          // enable multi-page scanning
    TWAIN_SetHideUI(fHideUI);           // enable/disable scanner UI
    
    if (TWAIN_OpenDefaultSource()) {
        
        CapabilityNegotiation(hwnd);
        
        while (TRUE) {
            HANDLE hdib = AcquireInMode(hwnd);            // try to acquire an image
            if (hdib) SetNewImage(hwnd, hdib);
            if (TWAIN_State() < TWAIN_SOURCE_ENABLED) {
                break;                                    // user closed scan dialog, end the loop
            }
            // otherwise, try another scan
        }
        TWAIN_CloseSource();
    }
    // Restore global multitransfer setting    
    TWAIN_SetMultiTransfer(bMulti);
} // MultiPageAcquire



//---------------------------------------------------------------------------

long FAR PASCAL  MainWndProc (HWND hwnd, WORD wMsg, WPARAM wP, LPARAM lP)
{
    HDC         	hDC;

	switch (wMsg) {

//-----------------------------------------------------------------
	case WM_INITMENU:
	{
		HMENU hmenu = (HMENU)wP;

        // enable or disable the file menu items
        UINT uEnable = MF_BYCOMMAND | (TWAIN_IsAvailable() ? MF_ENABLED : MF_GRAYED);
        if (insideAcquire != 0) {
            uEnable = MF_GRAYED;
        }
        EnableMenuItem(hmenu, TW_APP_ACQUIRE, uEnable);
        EnableMenuItem(hmenu, TW_APP_ACQUIRE_TO_CLPB, uEnable);
        EnableMenuItem(hmenu, TW_APP_ACQUIRE_MPS, uEnable);
        EnableMenuItem(hmenu, TW_APP_SELECT_SOURCE, uEnable);
        EnableMenuItem(hmenu, TW_APP_CLOSE_DS, (insideAcquire ? MF_ENABLED : MF_GRAYED));

        EnableMenuItem(hmenu, TW_APP_SAVEAS, MF_BYCOMMAND | (g_hdib ? MF_ENABLED : MF_GRAYED));
		CheckMenuItem(hmenu, TW_APP_BW, MF_BYCOMMAND | ((wPixTypes & TWAIN_BW) ? MF_CHECKED : MF_UNCHECKED));
		CheckMenuItem(hmenu, TW_APP_GRAYSCALE, MF_BYCOMMAND | ((wPixTypes & TWAIN_GRAY) ? MF_CHECKED : MF_UNCHECKED));
		CheckMenuItem(hmenu, TW_APP_RGB, MF_BYCOMMAND | ((wPixTypes & TWAIN_RGB) ? MF_CHECKED : MF_UNCHECKED));
		CheckMenuItem(hmenu, TW_APP_PALETTE, MF_BYCOMMAND | ((wPixTypes & TWAIN_PALETTE) ? MF_CHECKED : MF_UNCHECKED));
		CheckMenuItem(hmenu, TW_APP_ANYPIX, MF_BYCOMMAND | ((wPixTypes == TWAIN_ANYTYPE) ? MF_CHECKED : MF_UNCHECKED));
		CheckMenuItem(hmenu, TW_APP_SHOWUI, MF_BYCOMMAND | (fHideUI ? MF_UNCHECKED : MF_CHECKED));
		CheckMenuItem(hmenu, TW_APP_HIDEUI, MF_BYCOMMAND | (fHideUI ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_DISABLE_PARENT, MF_BYCOMMAND | (TWAIN_GetDisableParent() ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_MULTIXFER, MF_BYCOMMAND | (TWAIN_GetMultiTransfer() ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_DUPLEX, MF_BYCOMMAND | (bDuplex ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_INDICATORS, MF_BYCOMMAND | (bIndicators ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_FEEDEROFF, MF_BYCOMMAND | ((fFeeder<0) ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_FEEDERON, MF_BYCOMMAND | ((fFeeder>0) ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_AUTOFEED, MF_BYCOMMAND | (bAutoFeed ? MF_CHECKED : MF_UNCHECKED));

        CheckMenuItem(hmenu, TW_APP_RESNONE, MF_BYCOMMAND | (dResolution<0    ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_RES0,    MF_BYCOMMAND | (dResolution==0   ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_RES100,  MF_BYCOMMAND | (dResolution==100 ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_RES200,  MF_BYCOMMAND | (dResolution==200 ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_RES300,  MF_BYCOMMAND | (dResolution==300 ? MF_CHECKED : MF_UNCHECKED));
        CheckMenuItem(hmenu, TW_APP_RES600,  MF_BYCOMMAND | (dResolution==600 ? MF_CHECKED : MF_UNCHECKED));

		CheckMenuItem(hmenu, TW_APP_NATIVE, MF_BYCOMMAND | ((nXferMech == XFERMECH_NATIVE) ? MF_CHECKED : MF_UNCHECKED));
		CheckMenuItem(hmenu, TW_APP_MEMORY, MF_BYCOMMAND | ((nXferMech == XFERMECH_MEMORY) ? MF_CHECKED : MF_UNCHECKED));
		CheckMenuItem(hmenu, TW_APP_FILE, MF_BYCOMMAND | ((nXferMech == XFERMECH_FILE) ? MF_CHECKED : MF_UNCHECKED));

        CheckMenuItem(hmenu, TW_APP_SUPPRESS_MSGS, MF_BYCOMMAND | (bNoMessages ? MF_CHECKED : MF_UNCHECKED));

        return DefWindowProc (hwnd, wMsg, wP, lP);
	}
	
//-----------------------------------------------------------------
	case WM_DESTROY:
    	DiscardImage();
        TWAIN_UnloadSourceManager();
    	PostQuitMessage(0);
		return DefWindowProc (hwnd, wMsg, wP, lP);

//-----------------------------------------------------------------
	case WM_PALETTECHANGED:
		if ((HWND) wP == hwnd)
			return 0;				// this app changed palette: ignore

	// Otherwise, fall through to WM_QUERYNEWPALETTE.

	case WM_QUERYNEWPALETTE: {
		HPALETTE hpalT;
		UINT i;

		if (!hpal)
			return 0;				// no palette in effect

	    hDC = GetDC(hwnd);
    	hpalT = SelectPalette (hDC, hpal, FALSE);

		i = RealizePalette(hDC); // i == #entries that changed

	    SelectPalette (hDC, hpalT, FALSE);
    	ReleaseDC(hwnd, hDC);


    	// If any palette entries changed, repaint the window.

		if (i > 0) {
        	InvalidateRect(hwnd, NULL, TRUE);
        }

    	return i;
    } // WM_QUERYNEWPALETTE, WM_PALETTECHANGED

//-----------------------------------------------------------------
	case WM_PAINT: {
    	PAINTSTRUCT 	ps;
		hDC = BeginPaint(hwnd, &ps);
		if (hDC) {
			if (hpal) {
				SelectPalette (hDC, hpal, FALSE);
				RealizePalette (hDC);
			}   /* if */
			if (g_hdib) {
				int w = DIB_Width(g_hdib);
				int h = DIB_Height(g_hdib);
				HCURSOR hcurSave = SetCursor(hcurWait);
				DIB_DrawToDC(g_hdib, hDC, 0, 0, w, h, 0, 0);
				SetCursor(hcurSave);
			}
		}
        EndPaint(hwnd, &ps);
        break;
	} // WM_PAINT


//-----------------------------------------------------------------
	 case WM_COMMAND:

        switch (wP) {

			case TW_APP_ACQUIRE: {
                SynchronousAcquire(hwnd);
                break;
                                 } // case TW_APP_ACQUIRE

			case TW_APP_ACQUIRE_TO_CLPB: {
                insideAcquire++;
				if (!TWAIN_AcquireToClipboard(hwnd, TWAIN_ANYTYPE)) {
					LoadString(hInst, IDS_ERR_NOIMAGE, szMsg, sizeof szMsg);
					MessageBox(hwnd, szMsg, szAppName, MB_ICONINFORMATION | MB_OK);
				}
                insideAcquire--;
				break;
			}

            case TW_APP_ACQUIRE_MPS:
                MultiPageAcquire(hwnd);
                break;

			case TW_APP_SELECT_SOURCE:

				TWAIN_SelectImageSource(hwnd);
				break;


            case TW_APP_CLOSE_DS:
                TWAIN_CloseSource();
                break;

			case TW_APP_SAVEAS:
			{
				int result = TWAIN_WriteNativeToFilename(g_hdib, NULL);
				//	-1	user cancelled File Save dialog
				//	-x  other error
				if (result < -1) {
					LoadString(hInst, IDS_ERR_WRITE, szMsg, sizeof szMsg);
					MessageBox(hwnd, szMsg, szAppName, MB_ICONINFORMATION | MB_OK);
				}
				break;
			}

			case TW_APP_OPEN:
			{
				SetNewImage(hwnd, TWAIN_LoadNativeFromFilename(NULL));
				break;
			}

			case TW_APP_QUIT:
				DestroyWindow(hwnd);
				break;

//-----------------------------------------------------------------

            case TW_APP_ABOUT:
		    {
                // bring up the about dialog box
	            DialogBox (hInst, "TW_APP_ABOUTBOX", hwnd, AboutDlgProc);
	            break;
	        }

//-----------------------------------------------------------------

            case TW_APP_MEMORY:
                TWAIN_DisableSource();
            	nXferMech = XFERMECH_MEMORY;
	            break;
	        case TW_APP_NATIVE:
                TWAIN_DisableSource();
	        	nXferMech = XFERMECH_NATIVE;
	        	break;
	        case TW_APP_FILE:
                TWAIN_DisableSource();
            	nXferMech = XFERMECH_FILE;
	        	break;

//-----------------------------------------------------------------

            case TW_APP_BW:
                TWAIN_DisableSource();
            	wPixTypes ^= TWAIN_BW;
	            break;
	        case TW_APP_GRAYSCALE:
                TWAIN_DisableSource();
	        	wPixTypes ^= TWAIN_GRAY;
	        	break;
	        case TW_APP_RGB:
                TWAIN_DisableSource();
	        	wPixTypes ^= TWAIN_RGB;
	        	break;
	        case TW_APP_PALETTE:
                TWAIN_DisableSource();
	        	wPixTypes ^= TWAIN_PALETTE;
	        	break;
	        case TW_APP_ANYPIX:
                TWAIN_DisableSource();
	        	wPixTypes = TWAIN_ANYTYPE;
	        	break;

//-----------------------------------------------------------------

			case TW_APP_SHOWUI:
			case TW_APP_HIDEUI:
				fHideUI = (wP == TW_APP_HIDEUI);
                TWAIN_DisableSource();
				break;

//-----------------------------------------------------------------
            case TW_DISABLE_PARENT:
                TWAIN_DisableParent(!TWAIN_GetDisableParent());
                break;

//-----------------------------------------------------------------
            case TW_APP_MULTIXFER:
                TWAIN_DisableSource();
                TWAIN_SetMultiTransfer(!TWAIN_GetMultiTransfer());
                break;

//-----------------------------------------------------------------
            case TW_APP_DUPLEX:
                TWAIN_DisableSource();
                bDuplex = !bDuplex;
                TWAIN_EnableDuplex(bDuplex);
                break;

//-----------------------------------------------------------------
            case TW_APP_RESNONE:
                dResolution = -1.0;
                TWAIN_DisableSource();
                TWAIN_SetCurrentResolution(dResolution);
                break;
            case TW_APP_RES0:
                dResolution = 0.0;
                TWAIN_DisableSource();
                TWAIN_SetCurrentResolution(dResolution);
                break;
            case TW_APP_RES100:
                dResolution = 100.0;
                TWAIN_DisableSource();
                TWAIN_SetCurrentResolution(dResolution);
                break;
            case TW_APP_RES200:
                dResolution = 200.0;
                TWAIN_DisableSource();
                TWAIN_SetCurrentResolution(dResolution);
                break;
            case TW_APP_RES300:
                dResolution = 300.0;
                TWAIN_DisableSource();
                TWAIN_SetCurrentResolution(dResolution);
                break;
            case TW_APP_RES600:
                dResolution = 600.0;
                TWAIN_DisableSource();
                TWAIN_SetCurrentResolution(dResolution);
                break;

//-----------------------------------------------------------------
            case TW_APP_INDICATORS:
                bIndicators = !bIndicators;
                TWAIN_SetIndicators(bIndicators);
                break;

//-----------------------------------------------------------------
            case TW_APP_SUPPRESS_MSGS:
                bNoMessages = !bNoMessages;
                TWAIN_SuppressErrorMessages(bNoMessages);
                break;

//-----------------------------------------------------------------
            case TW_APP_FEEDEROFF:
                // Disable document feeder
                TWAIN_DisableSource();
                fFeeder = -1;
                bAutoFeed = FALSE;
                break;
            case TW_APP_FEEDERON:
                // Enable document feeder
                TWAIN_DisableSource();
                fFeeder = 1;
                break;
            case TW_APP_AUTOFEED:
                // Toggle auto-feed, enable document feeder
                TWAIN_DisableSource();
                fFeeder = 1;
                bAutoFeed = !bAutoFeed;
                break;

//-----------------------------------------------------------------
		  default:
 	         break;
        }	/* switch */
        break;

	default:
		return DefWindowProc (hwnd, wMsg, wP, lP);
	}	/* switch */
	return 0L ;
}	// MainWndProc



//---------------------------------------------------------------------------

void ResizeWindow(HWND hwnd, HANDLE hdib)
{
	HDC	hDC = GetDC(hwnd);
	if (hDC) {
		int xMin = LOWORD(GetTabbedTextExtent(hDC, "Twerp Sample Title", 18, 0, NULL));
        RECT		Rectangle;
    	Rectangle.left = 0;
    	Rectangle.top  = 0;
    	Rectangle.right  = max(DIB_Width(hdib), xMin);
    	Rectangle.bottom = DIB_Height(hdib);

		// AdjustWindowRect fails if you have a multiline menu.
		AdjustWindowRect (&Rectangle,  WS_OVERLAPPEDWINDOW, TRUE);

    	SetWindowPos (hwnd, (HWND)NULL, 0, 0,
            		  Rectangle.right - Rectangle.left,
					  Rectangle.bottom - Rectangle.top,
					  SWP_NOMOVE | SWP_NOZORDER);

    	ReleaseDC(hwnd, hDC);
    }
} // end ResizeWindow


//---------------------------------------------------------------------------

static void LoadSourceList(HWND hlb)
{
    if (hlb && TWAIN_IsAvailable() && TWAIN_GetSourceList()) {
        CHAR szName[256];
        while (TWAIN_GetNextSourceName(szName)) {
            SendMessage(hlb, LB_ADDSTRING, 0, (LPARAM)szName);
        }
        if (TWAIN_GetDefaultSourceName(szName)) {
            SendMessage(hlb, LB_SELECTSTRING, (WPARAM)-1, (LPARAM)szName);
        }
    }
} // LoadSourceList


BOOL FAR PASCAL AboutDlgProc(HWND hDlg, UINT message, WPARAM wP, LPARAM lParam)
// Call-back function (dialog procedure) for 'About' box.
// On OK or Cancel, closes the dialog.
{
	int nVer = TWAIN_EasyVersion();
	char szBuff[100];

	lParam=lParam;								// suppress no-use warning
	switch (message){
		case WM_INITDIALOG:						// message: initialize dialog box
			LoadString(hInst, IDS_EZT_VER, szMsg, sizeof szMsg);
			wsprintf(szBuff, szMsg, nVer/100, nVer % 100, TWAIN_BuildName());
			SetDlgItemText(hDlg, IDEZT_VER, szBuff);    
			LoadString(hInst, (TWAIN_IsAvailable() ? IDS_TWAIN_PRESENT : IDS_NO_TWAIN),
			           szBuff, sizeof szBuff);
			SetDlgItemText(hDlg, IDTWAIN_AVAIL, szBuff);
            LoadSourceList(GetDlgItem(hDlg, IDC_SLIST));
			return (TRUE);

		case WM_COMMAND:						// message: received a command
			if ((wP == IDOK) || (wP == IDCANCEL)) {
												// OK button or System menu close
				EndDialog(hDlg, TRUE);			// Exits the dialog box
				return (TRUE);
			}
			break;
		default:
			break;
	}

    return (FALSE);            // Didn't process a message
} // AboutDlgProc


//---------------------------------------------------------------------------

VOID DiscardImage (VOID)
// delete/free global palette, and dib, as necessary.
{

	if (hpal) {
		DeleteObject(hpal);
		hpal = NULL;
	}
	if (g_hdib) {
		TWAIN_FreeNative(g_hdib);
		g_hdib = NULL;
	}
} // DiscardImage

void SetNewImage(HWND hwnd, HANDLE hdib)
{
    DiscardImage();
    g_hdib = hdib;
    if (g_hdib) {
        // compute or guess a palette to use for display
        hpal = DIB_CreatePalette(g_hdib);
        // size the window to just contain the image
        ResizeWindow(hwnd, g_hdib);
    }
    // force repaint of window
    InvalidateRect(hwnd, NULL, TRUE);
    UpdateWindow(hwnd);
}





